@@ -32,6 +32,8 @@ from .api_pb2 import (  # type: ignore
     BluetoothGATTWriteResponse,
     BluetoothLEAdvertisementResponse,
     BluetoothLERawAdvertisementsResponse,
+    BluetoothScannerSetModeRequest,
+    BluetoothScannerStateResponse,
     ButtonCommandRequest,
     CameraImageRequest,
     CameraImageResponse,
@@ -52,6 +54,8 @@ from .api_pb2 import (  # type: ignore
     ListEntitiesServicesResponse,
     LockCommandRequest,
     MediaPlayerCommandRequest,
+    NoiseEncryptionSetKeyRequest,
+    NoiseEncryptionSetKeyResponse,
     NumberCommandRequest,
     SelectCommandRequest,
     SirenCommandRequest,
@@ -82,18 +86,20 @@ from .api_pb2 import (  # type: ignore
     VoiceAssistantSetConfiguration,
     VoiceAssistantTimerEventResponse,
 )
-from .client_callbacks import (
+from .client_base import (
+    APIClientBase,
     on_bluetooth_connections_free_response,
     on_bluetooth_device_connection_response,
     on_bluetooth_gatt_notify_data_response,
     on_bluetooth_handle_message,
     on_bluetooth_le_advertising_response,
     on_bluetooth_message_types,
+    on_bluetooth_scanner_state_response,
     on_home_assistant_service_response,
     on_state_msg,
     on_subscribe_home_assistant_state_response,
 )
-from .connection import APIConnection, ConnectionParams, handle_timeout
+from .connection import APIConnection, ConnectionParams, handle_timeout  # noqa: F401
 from .core import (
     APIConnectionError,
     BluetoothConnectionDroppedError,
@@ -114,6 +120,8 @@ from .model import (
     BluetoothLEAdvertisement,
     BluetoothProxyFeature,
     BluetoothProxySubscriptionFlag,
+    BluetoothScannerMode,
+    BluetoothScannerStateResponse as BluetoothScannerStateResponseModel,
     ClimateFanMode,
     ClimateMode,
     ClimatePreset,
@@ -129,6 +137,7 @@ from .model import (
     LockCommand,
     LogLevel,
     MediaPlayerCommand,
+    NoiseEncryptionSetKeyResponse as NoiseEncryptionSetKeyResponseModel,
     UpdateCommand,
     UserService,
     UserServiceArgType,
@@ -146,25 +155,13 @@ from .model_conversions import (
     LIST_ENTITIES_SERVICES_RESPONSE_TYPES,
     SUBSCRIBE_STATES_RESPONSE_TYPES,
 )
-from .util import build_log_name, create_eager_task
-from .zeroconf import ZeroconfInstanceType, ZeroconfManager
+from .util import create_eager_task
 
 _LOGGER = logging.getLogger(__name__)
 
 DEFAULT_BLE_TIMEOUT = 30.0
 DEFAULT_BLE_DISCONNECT_TIMEOUT = 20.0
 
-# We send a ping every 20 seconds, and the timeout ratio is 4.5x the
-# ping interval. This means that if we don't receive a ping for 90.0
-# seconds, we'll consider the connection dead and reconnect.
-#
-# This was chosen because the 20s is around the expected time for a
-# device to reboot and reconnect to wifi, and 90 seconds is the absolute
-# maximum time a device can take to respond when its behind + the WiFi
-# connection is poor.
-KEEP_ALIVE_FREQUENCY = 20.0
-
-
 SUBSCRIBE_STATES_MSG_TYPES = (*SUBSCRIBE_STATES_RESPONSE_TYPES, CameraImageResponse)
 
 LIST_ENTITIES_MSG_TYPES = (
@@ -192,18 +189,8 @@ ExecuteServiceDataType = dict[
 ]
 
 
-def _stringify_or_none(value: str | None) -> str | None:
-    """Convert a string like object to a str or None.
-
-    The noise_psk is sometimes passed into
-    the client as an Estr, but we want to pass it
-    to the API as a string or None.
-    """
-    return None if value is None else str(value)
-
-
 # pylint: disable=too-many-public-methods
-class APIClient:
+class APIClient(APIClientBase):
     """The ESPHome API client.
 
     This class is the main entrypoint for interacting with the API.
@@ -213,107 +200,6 @@ class APIClient:
     if the connection is lost.
     """
 
-    __slots__ = (
-        "_debug_enabled",
-        "_params",
-        "_connection",
-        "cached_name",
-        "_background_tasks",
-        "_loop",
-        "log_name",
-    )
-
-    def __init__(
-        self,
-        address: str,
-        port: int,
-        password: str | None,
-        *,
-        client_info: str = "aioesphomeapi",
-        keepalive: float = KEEP_ALIVE_FREQUENCY,
-        zeroconf_instance: ZeroconfInstanceType | None = None,
-        noise_psk: str | None = None,
-        expected_name: str | None = None,
-        addresses: list[str] | None = None,
-    ) -> None:
-        """Create a client, this object is shared across sessions.
-
-        :param address: The address to connect to; for example an IP address
-          or .local name for mDNS lookup.
-        :param port: The port to connect to
-        :param password: Optional password to send to the device for authentication
-        :param client_info: User Agent string to send.
-        :param keepalive: The keepalive time in seconds (ping interval) for detecting stale connections.
-            Every keepalive seconds a ping is sent, if no pong is received the connection is closed.
-        :param zeroconf_instance: Pass a zeroconf instance to use if an mDNS lookup is necessary.
-        :param noise_psk: Encryption preshared key for noise transport encrypted sessions.
-        :param expected_name: Require the devices name to match the given expected name.
-            Can be used to prevent accidentally connecting to a different device if
-            IP passed as address but DHCP reassigned IP.
-        :param addresses: Optional list of IP addresses to connect to which takes
-            precedence over the address parameter. This is most commonly used when
-            the device has dual stack IPv4 and IPv6 addresses and you do not know
-            which one to connect to.
-        """
-        self._debug_enabled = _LOGGER.isEnabledFor(logging.DEBUG)
-        self._params = ConnectionParams(
-            addresses=addresses if addresses else [str(address)],
-            port=port,
-            password=password,
-            client_info=client_info,
-            keepalive=keepalive,
-            zeroconf_manager=ZeroconfManager(zeroconf_instance),
-            # treat empty '' psk string as missing (like password)
-            noise_psk=_stringify_or_none(noise_psk) or None,
-            expected_name=_stringify_or_none(expected_name) or None,
-        )
-        self._connection: APIConnection | None = None
-        self.cached_name: str | None = None
-        self._background_tasks: set[asyncio.Task[Any]] = set()
-        self._loop = asyncio.get_event_loop()
-        self._set_log_name()
-
-    def set_debug(self, enabled: bool) -> None:
-        """Enable debug logging."""
-        self._debug_enabled = enabled
-        if self._connection:
-            self._connection.set_debug(enabled)
-
-    @property
-    def zeroconf_manager(self) -> ZeroconfManager:
-        return self._params.zeroconf_manager
-
-    @property
-    def expected_name(self) -> str | None:
-        return self._params.expected_name
-
-    @expected_name.setter
-    def expected_name(self, value: str | None) -> None:
-        self._params.expected_name = value
-
-    @property
-    def address(self) -> str:
-        return self._params.addresses[0]
-
-    def _set_log_name(self) -> None:
-        """Set the log name of the device."""
-        connected_address: str | None = None
-        if self._connection and self._connection.connected_address:
-            connected_address = self._connection.connected_address
-        self.log_name = build_log_name(
-            self.cached_name,
-            self._params.addresses,
-            connected_address,
-        )
-        if self._connection:
-            self._connection.set_log_name(self.log_name)
-
-    def set_cached_name_if_unset(self, name: str) -> None:
-        """Set the cached name of the device if not set."""
-        if not self.cached_name:
-            self.cached_name = name
-            self._set_log_name()
-
     async def connect(
         self,
         on_stop: Callable[[bool], Coroutine[Any, Any, None]] | None = None,
@@ -380,17 +266,6 @@ class APIClient:
         else:
             await self._connection.disconnect()
 
-    def _get_connection(self) -> APIConnection:
-        connection = self._connection
-        if not connection:
-            raise APIConnectionError(f"Not connected to {self.log_name}!")
-        if not connection.is_connected:
-            raise APIConnectionError(
-                f"Authenticated connection not ready yet for {self.log_name}; "
-                f"current state is {connection.connection_state}!"
-            )
-        return connection
-
     async def device_info(self) -> DeviceInfo:
         resp = await self._get_connection().send_message_await_response(
             DeviceInfoRequest(), DeviceInfoResponse
@@ -399,11 +274,6 @@ class APIClient:
         self._set_name_from_device(info.name)
         return info
 
-    def _set_name_from_device(self, name: str) -> None:
-        """Set the name from a DeviceInfo message."""
-        self.cached_name = name
-        self._set_log_name()
-
     async def list_entities_services(
         self,
     ) -> tuple[list[EntityInfo], list[UserService]]:
@@ -439,13 +309,25 @@ class APIClient:
         on_log: Callable[[SubscribeLogsResponse], None],
         log_level: LogLevel | None = None,
         dump_config: bool | None = None,
-    ) -> None:
+    ) -> Callable[[], None]:
+        """Subscribe to logs.
+
+        Returns a callable that can be called to stop
+        the callbacks. Calling the callable only
+        stops the callbacks. The device will still
+        send logs until the logging level is set to
+        LogLevel.LOG_LEVEL_NONE.
+
+        To stop the device sending logs completely, call
+        with log_level=LogLevel.LOG_LEVEL_NONE, and call the returned
+        callable to unsubscribe.
+        """
         req = SubscribeLogsRequest()
         if log_level is not None:
             req.level = log_level
         if dump_config is not None:
             req.dump_config = dump_config
-        self._get_connection().send_message_callback_response(
+        return self._get_connection().send_message_callback_response(
             req, on_log, (SubscribeLogsResponse,)
         )
 
@@ -521,7 +403,8 @@ class APIClient:
         return partial(self._unsub_bluetooth_advertisements, unsub_callback)
 
     def subscribe_bluetooth_connections_free(
-        self, on_bluetooth_connections_free_update: Callable[[int, int], None]
+        self,
+        on_bluetooth_connections_free_update: Callable[[int, int, list[int]], None],
     ) -> Callable[[], None]:
         return self._get_connection().send_message_callback_response(
             SubscribeBluetoothConnectionsFreeRequest(),
@@ -532,6 +415,25 @@ class APIClient:
             (BluetoothConnectionsFreeResponse,),
         )
 
+    def subscribe_bluetooth_scanner_state(
+        self,
+        on_bluetooth_scanner_state: Callable[
+            [BluetoothScannerStateResponseModel], None
+        ],
+    ) -> Callable[[], None]:
+        """Subscribe to Bluetooth scanner state updates."""
+        return self._get_connection().add_message_callback(
+            partial(
+                on_bluetooth_scanner_state_response,
+                on_bluetooth_scanner_state,
+            ),
+            (BluetoothScannerStateResponse,),
+        )
+
+    def bluetooth_scanner_set_mode(self, mode: BluetoothScannerMode) -> None:
+        """Set the Bluetooth scanner mode."""
+        self._get_connection().send_message(BluetoothScannerSetModeRequest(mode=mode))
+
     async def bluetooth_device_connect(  # pylint: disable=too-many-locals, too-many-branches
         self,
         address: int,
@@ -1069,10 +971,10 @@ class APIClient:
             req.warm_white = warm_white
         if transition_length is not None:
             req.has_transition_length = True
-            req.transition_length = int(round(transition_length * 1000))
+            req.transition_length = round(transition_length * 1000)
         if flash_length is not None:
             req.has_flash_length = True
-            req.flash_length = int(round(flash_length * 1000))
+            req.flash_length = round(flash_length * 1000)
         if effect is not None:
             req.has_effect = True
             req.effect = effect
@@ -1123,8 +1025,8 @@ class APIClient:
             if TYPE_CHECKING:
                 assert apiv is not None
             if apiv < APIVersion(1, 5):
-                req.has_legacy_away = True
-                req.legacy_away = preset == ClimatePreset.AWAY
+                req.unused_has_legacy_away = True
+                req.unused_legacy_away = preset == ClimatePreset.AWAY
             else:
                 req.has_preset = True
                 req.preset = preset
@@ -1288,12 +1190,6 @@ class APIClient:
     def request_image_stream(self) -> None:
         self._request_image(stream=True)
 
-    @property
-    def api_version(self) -> APIVersion | None:
-        if self._connection is None:
-            return None
-        return self._connection.api_version
-
     def subscribe_voice_assistant(
         self,
         *,
@@ -1409,8 +1305,6 @@ class APIClient:
             )
 
         def unsub() -> None:
-            nonlocal start_task
-
             if self._connection is not None:
                 for remove_callback in remove_callbacks:
                     remove_callback()
@@ -1423,12 +1317,6 @@ class APIClient:
 
         return unsub
 
-    def _create_background_task(self, coro: Coroutine[Any, Any, None]) -> None:
-        """Create a background task and add it to the background tasks set."""
-        task = create_eager_task(coro)
-        self._background_tasks.add(task)
-        task.add_done_callback(self._background_tasks.discard)
-
     def send_voice_assistant_event(
         self, event_type: VoiceAssistantEventType, data: dict[str, str] | None
     ) -> None:
@@ -1471,9 +1359,16 @@ class APIClient:
         media_id: str,
         timeout: float,
         text: str = "",
+        preannounce_media_id: str = "",
+        start_conversation: bool = False,
     ) -> VoiceAssistantAnnounceFinishedModel:
         resp = await self._get_connection().send_message_await_response(
-            VoiceAssistantAnnounceRequest(media_id=media_id, text=text),
+            VoiceAssistantAnnounceRequest(
+                media_id=media_id,
+                text=text,
+                preannounce_media_id=preannounce_media_id,
+                start_conversation=start_conversation,
+            ),
             VoiceAssistantAnnounceFinished,
             timeout,
         )
@@ -1505,3 +1400,14 @@ class APIClient:
         if code is not None:
             req.code = code
         self._get_connection().send_message(req)
+
+    async def noise_encryption_set_key(
+        self,
+        key: bytes,
+    ) -> bool:
+        """Set the noise encryption key."""
+        req = NoiseEncryptionSetKeyRequest(key=key)
+        resp = await self._get_connection().send_message_await_response(
+            req, NoiseEncryptionSetKeyResponse
+        )
+        return NoiseEncryptionSetKeyResponseModel.from_pb(resp).success
