@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 from collections.abc import Iterable
+import contextlib
 from dataclasses import asdict, dataclass, field, fields
 import enum
 from functools import cache, lru_cache, partial
@@ -50,10 +51,8 @@ class APIIntEnum(enum.IntEnum):
     def convert_list(cls: type[_T], value: list[int]) -> list[_T]:
         ret = []
         for x in value:
-            try:
+            with contextlib.suppress(ValueError):
                 ret.append(cls(x))
-            except ValueError:
-                pass
         return ret
 
 
@@ -115,6 +114,7 @@ class BluetoothProxyFeature(enum.IntFlag):
     PAIRING = 1 << 3
     CACHE_CLEARING = 1 << 4
     RAW_ADVERTISEMENTS = 1 << 5
+    FEATURE_STATE_AND_MODE = 1 << 6
 
 
 class BluetoothProxySubscriptionFlag(enum.IntFlag):
@@ -127,6 +127,7 @@ class VoiceAssistantFeature(enum.IntFlag):
     API_AUDIO = 1 << 2
     TIMERS = 1 << 3
     ANNOUNCE = 1 << 4
+    START_CONVERSATION = 1 << 5
 
 
 class VoiceAssistantSubscriptionFlag(enum.IntFlag):
@@ -152,6 +153,7 @@ class DeviceInfo(APIModelBase):
     legacy_bluetooth_proxy_version: int = 0
     bluetooth_proxy_feature_flags: int = 0
     suggested_area: str = ""
+    bluetooth_mac_address: str = ""
 
     def bluetooth_proxy_feature_flags_compat(self, api_version: APIVersion) -> int:
         if api_version < APIVersion(1, 9):
@@ -283,7 +285,7 @@ class FanInfo(EntityInfo):
     supports_oscillation: bool = False
     supports_speed: bool = False
     supports_direction: bool = False
-    supported_speed_levels: int = 0
+    supported_speed_count: int = 0
     supported_preset_modes: list[str] = converter_field(
         default_factory=list, converter=list
     )
@@ -324,10 +326,23 @@ class LightColorCapability(enum.IntFlag):
     RGB = 1 << 5
 
 
+class ColorMode(APIIntEnum):
+    UNKNOWN = 0
+    ON_OFF = 1
+    BRIGHTNESS = 2
+    WHITE = 7
+    COLOR_TEMPERATURE = 11
+    COLD_WARM_WHITE = 19
+    RGB = 35
+    RGB_WHITE = 39
+    RGB_COLOR_TEMPERATURE = 47
+    RGB_COLD_WARM_WHITE = 51
+
+
 @_frozen_dataclass_decorator
 class LightInfo(EntityInfo):
-    supported_color_modes: list[int] = converter_field(
-        default_factory=list, converter=list
+    supported_color_modes: list[ColorMode] = converter_field(
+        default_factory=list, converter=ColorMode.convert_list
     )
     min_mireds: float = converter_field(
         default=0.0, converter=fix_float_single_double_conversion
@@ -343,7 +358,7 @@ class LightInfo(EntityInfo):
     legacy_supports_white_value: bool = False
     legacy_supports_color_temperature: bool = False
 
-    def supported_color_modes_compat(self, api_version: APIVersion) -> list[int]:
+    def supported_color_modes_compat(self, api_version: APIVersion) -> list[ColorMode]:
         if api_version < APIVersion(1, 6):
             key = (
                 self.legacy_supports_brightness,
@@ -389,7 +404,7 @@ class LightInfo(EntityInfo):
                 ],
             }
 
-            return cast(list[int], modes_map[key]) if key in modes_map else []
+            return cast(list[ColorMode], modes_map[key]) if key in modes_map else []
 
         return self.supported_color_modes
 
@@ -400,7 +415,7 @@ class LightState(EntityState):
     brightness: float = converter_field(
         default=0.0, converter=fix_float_single_double_conversion
     )
-    color_mode: int = 0
+    color_mode: ColorMode = ColorMode.UNKNOWN
     color_brightness: float = converter_field(
         default=0.0, converter=fix_float_single_double_conversion
     )
@@ -451,7 +466,7 @@ class SensorInfo(EntityInfo):
     state_class: SensorStateClass | None = converter_field(
         default=SensorStateClass.NONE, converter=SensorStateClass.convert
     )
-    last_reset_type: LastResetType | None = converter_field(
+    legacy_last_reset_type: LastResetType | None = converter_field(
         default=LastResetType.NONE, converter=LastResetType.convert
     )
 
@@ -619,7 +634,7 @@ class ClimateState(EntityState):
     target_temperature_high: float = converter_field(
         default=0.0, converter=fix_float_single_double_conversion
     )
-    legacy_away: bool = False
+    unused_legacy_away: bool = False
     fan_mode: ClimateFanMode | None = converter_field(
         default=ClimateFanMode.ON, converter=ClimateFanMode.convert
     )
@@ -636,7 +651,7 @@ class ClimateState(EntityState):
 
     def preset_compat(self, api_version: APIVersion) -> ClimatePreset | None:
         if api_version < APIVersion(1, 5):
-            return ClimatePreset.AWAY if self.legacy_away else ClimatePreset.HOME
+            return ClimatePreset.AWAY if self.unused_legacy_away else ClimatePreset.HOME
         return self.preset
 
 
@@ -1263,6 +1278,30 @@ class BluetoothDeviceRequestType(APIIntEnum):
     CLEAR_CACHE = 6
 
 
+class BluetoothScannerState(APIIntEnum):
+    IDLE = 0
+    STARTING = 1
+    RUNNING = 2
+    FAILED = 3
+    STOPPING = 4
+    STOPPED = 5
+
+
+class BluetoothScannerMode(APIIntEnum):
+    PASSIVE = 0
+    ACTIVE = 1
+
+
+@_frozen_dataclass_decorator
+class BluetoothScannerStateResponse(APIModelBase):
+    state: BluetoothScannerState | None = converter_field(
+        default=BluetoothScannerState.IDLE, converter=BluetoothScannerState.convert
+    )
+    mode: BluetoothScannerMode | None = converter_field(
+        default=BluetoothScannerMode.PASSIVE, converter=BluetoothScannerMode.convert
+    )
+
+
 class VoiceAssistantCommandFlag(enum.IntFlag):
     USE_VAD = 1 << 0
     USE_WAKE_WORD = 1 << 1
@@ -1334,6 +1373,16 @@ class VoiceAssistantSetConfiguration(APIModelBase):
     active_wake_words: list[int] = converter_field(default_factory=list, converter=list)
 
 
+@_frozen_dataclass_decorator
+class NoiseEncryptionSetKeyRequest(APIModelBase):
+    key: bytes = field(default_factory=bytes)  # pylint: disable=invalid-field-call
+
+
+@_frozen_dataclass_decorator
+class NoiseEncryptionSetKeyResponse(APIModelBase):
+    success: bool = False
+
+
 class LogLevel(APIIntEnum):
     LOG_LEVEL_NONE = 0
     LOG_LEVEL_ERROR = 1
